from bloonix.heaven import H
from bloonix.heaven.plugin.crud import Validate

class Controller(Validate):
    def startup(self):
        self.db = H.model.db.chart_view
        H.route("/chart-views", "list")
        H.route("/chart-views/:chart_id", "show")
        H.route("/chart-views/create", "create")
        H.route("/chart-views/options", "options")
        H.route("/chart-views/:chart_id/update", "update")
        H.route("/chart-views/:chart_id/delete", "delete")
        H.route("/chart-views/:chart_id/options", "options")
        H.route("/chart-views/:chart_id/update-data", "update_data")

    def auto(self, opts):
        if "chart_id" in opts:
            value = self.db.get_chart_view(
                chart_view_id=opts["chart_id"],
                user_id=H.user["id"])
            if not value:
                return H.plugin.error.set("err-600")
            H.cache[self.db.table] = value
        return True

    def list(self, opts):
        H.stash.data = self.db.get_all(
            host_id=opts.get("host_id", None),
            user_id=H.user["id"])

    def show(self, opts):
        H.stash.data = H.cache[self.db.table]

    def options(self, opts):
        H.stash.data["options"] = self.db.validator.get_options()
        if "chart_id" in opts:
            H.stash.data["values"] = H.cache.get(self.db.table)
        else:
            H.stash.data["values"] = self.db.validator.get_defaults()

    def create(self, opts):
        data = self.validate(opts=opts)
        if not data:
            return
        data["user_id"] = H.user["id"]
        data["host_id"] = opts.get("host_id", None)
        try:
            data = self.db.create_and_get(data)
        except BloonixDuplicationError:
            return self.raise_on_dup_error()
        except Exception:
            raise
        H.plugin.audit.create(
            target=self.db.table,
            new_data=data)
        H.stash.data = data

    def update(self, opts):
        data = self.validate(opts=opts, ignore_missing=True)
        if not data:
            return
        try:
            self.db.update_by_id_and_user_id(opts["chart_id"], H.user["id"], data)
        except BloonixDuplicationError:
            return self.raise_on_dup_error()
        except Exception:
            raise
        H.plugin.audit.update(
            target=self.db.table,
            old_data=H.cache.get(self.db.table),
            new_data=data)

    def delete(self, opts):
        if not H.plugin.token.check():
            return
        ret = self.db.delete_by_id_and_user_id(opts["chart_id"], H.user["id"])
        H.plugin.audit.delete(
            target=self.db.table,
            old_data=H.cache.get(self.db.table))
        return ret

    def update_data(self, opts):
        charts = H.req.param("charts")
        options = H.req.param("options")
        # validation ???
        self.db.update_data_by_id_and_user_id(
            opts["chart_id"],
            H.user["id"],
            {"charts": charts, "options": options})
