import re
import time
import copy
from bloonix.heaven import H
from bloonix import timeperiod
from bloonix import utils
from bloonix import timezone
from bloonix import rx

class Controller:
    def startup(self):
        self.dbh = H.model.db.host_downtime
        self.dbs = H.model.db.service_downtime
        _type = ":type(fixtime|timeslice|preset)"
        H.route("/config/hosts/:id/downtimes", "list_host_downtimes")
        H.route("/config/hosts/:id/downtimes/add/%s" % _type, "add_host_downtime")
        H.route("/config/hosts/downtimes/options/%s" % _type, "host_downtime_options")
        H.route("/config/hosts/downtimes/validate/%s" % _type, "host_downtime_validate")
        H.route("/config/hosts/:id/downtimes/options/%s" % _type, "host_downtime_options")
        H.route("/config/hosts/:id/downtimes/:host_downtime_id/remove", "remove_host_downtime")
        H.route("/config/hosts/:id/downtimes/services", "list_service_downtimes")
        H.route("/config/hosts/:id/downtimes/services/add/%s" % _type, "add_service_downtime")
        H.route("/config/hosts/:id/downtimes/services/options/%s" % _type, "service_downtime_options")
        H.route("/config/hosts/:id/downtimes/services/:service_downtime_id/remove", "remove_service_downtime")

    def auto(self, opts):
        if "host_downtime_id" in opts or "service_downtime_id" in opts:
            if "host_downtime_id" in opts:
                H.cache["downtime"] = self.dbh.by_id_and_host_id(opts["host_downtime_id"], opts["id"])
            elif "service_downtime_id" in opts:
                H.cache["downtime"] = self.dbs.by_id_and_host_id(opts["service_downtime_id"], opts["id"])
                if not H.cache["downtime"]:
                    return H.plugin.error.set("err-600")
        return True

    def list_host_downtimes(self, opts):
        H.stash.data = self.dbh.by_host_id(opts["id"])
        H.stash.total = len(H.stash.data)

    def list_service_downtimes(self, opts):
        H.stash.data = self.dbs.by_host_id(opts["id"])
        H.stash.total = len(H.stash.data)

    def host_downtime_options(self, opts):
        v = self.dbh.init_validator(opts["type"])
        H.stash.data["options"] = v.get_options()
        H.stash.data["values"] = v.get_defaults()

    def host_downtime_validate(self, opts):
        data = self.validate(opts["type"])
        if not data:
            return
        H.stash.data = data

    def service_downtime_options(self, opts):
        v = self.dbh.init_validator(opts["type"], opts["id"])
        H.stash.data["options"] = v.get_options()
        H.stash.data["values"] = v.get_defaults()

    def validate(self, downtime_type, host_id=None):
        v = self.dbh.init_validator(downtime_type, host_id)
        res = v.validate()
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)
        if "preset" in res.data:
            res.data["begin"] = int(time.time())
            res.data["end"] = res.data["begin"] + res.data["preset"]
            del res.data["preset"]
        return res.data

    def add_host_downtime(self, opts):
        if not H.plugin.token.check():
            return

        data = self.validate(opts["type"])
        if not data:
            return

        data["host_id"] = opts["id"]
        self.dbh.create(data)
        H.plugin.audit.create(
            target="host_downtime",
            new_data=data)

    def add_service_downtime(self, opts):
        if not H.plugin.token.check():
            return

        data = self.validate(opts["type"], opts["id"])
        if not data:
            return

        data["host_id"] = opts["id"]
        service_ids = data["service_ids"]
        del data["service_ids"]
        for service_id in service_ids:
            dcopy = copy.deepcopy(data)
            dcopy["service_id"] = service_id
            self.dbs.create(dcopy)
            H.plugin.audit.create(
                target="service_downtime",
                new_data=dcopy)

    def remove_host_downtime(self, opts):
        if not H.plugin.token.check():
            return
        self.dbh.delete_by_id(opts["host_downtime_id"])
        H.plugin.audit.delete(
            target="host_downtime",
            old_data=H.cache["downtime"])

    def remove_service_downtime(self, opts):
        if not H.plugin.token.check():
            return
        self.dbs.delete_by_id(opts["service_downtime_id"])
        H.plugin.audit.delete(
            target="service_downtime",
            old_data=H.cache["downtime"])
