from bloonix.heaven import H
import re

class Controller:
    def startup(self):
        H.route("/config/hosts/:id/templates/member", "member_of_templates")
        H.route("/config/hosts/:id/templates/no-member", "no_member_of_templates")
        H.route("/config/hosts/:id/templates/:template_id/add", "add_to_templates")
        H.route("/config/hosts/:id/templates/:template_id/remove", "remove_from_templates")
        H.route("/config/hosts/:id/contactgroups/member", "member_of_contactgroups")
        H.route("/config/hosts/:id/contactgroups/no-member", "no_member_of_contactgroups")
        H.route("/config/hosts/:id/contactgroups/:contactgroup_id/add", "add_to_contactgroups")
        H.route("/config/hosts/:id/contactgroups/:contactgroup_id/remove", "remove_from_contactgroups")
        H.route("/config/hosts/:id/groups/member", "member_of_groups")
        H.route("/config/hosts/:id/groups/no-member", "no_member_of_groups")
        H.route("/config/hosts/:id/groups/:group_id/add", "add_to_groups")
        H.route("/config/hosts/:id/groups/:group_id/remove", "remove_from_groups")
        H.route("/config/hosts/:id/location-groups/member", "member_of_location_groups")
        H.route("/config/hosts/:id/location-groups/no-member", "no_member_of_location_groups")
        H.route("/config/hosts/:id/location-groups/:location_group_id/add", "add_to_location_groups")
        H.route("/config/hosts/:id/location-groups/:location_group_id/remove", "remove_from_location_groups")

    def auto(self, opts):
        if "templates_id" in opts:
            if not H.model.db.template.get(opts["template_id"]):
                return H.plugin.error.set("err-600")
        if "group_id" in opts:
            if not H.model.db.group.get(opts["group_id"]):
                return H.plugin.error.set("err-600")
        if "location_group_id" in opts:
            if not H.model.db.location_group.get(opts["location_group_id"]):
                return H.plugin.error.set("err-600")
        return True

    def member_of_templates(self, opts):
        ret = H.model.db.template.query(
            query=H.req.get_query_data(),
            count="id",
            table=["template", ["id", "name", "tags"]],
            join=[
                "inner", [
                    "host_template",
                    "template.id",
                    "host_template.template_id"
                ]
            ],
            condition=["where", ["host_template.host_id", opts["id"]]],
            order=["asc", "template.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def no_member_of_templates(self, opts):
        ret = H.model.db.template.query(
            query=H.req.get_query_data(),
            count="id",
            table=["template", ["id", "name", "tags"]],
            condition=[
                "where", ["template.id", "not in", {
                    "table": "template",
                    "columns": "id",
                    "join": [
                        "inner", ["host_template", "template.id", "host_template.template_id"],
                    ],
                    "condition": [
                        "where", ["host_template.host_id", opts["id"]],
                    ]
                }]
            ],
            order=["asc", "template.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def add_to_templates(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_template.add_host(opts["template_id"], opts["id"])

    def remove_from_templates(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_template.remove_host(opts["template_id"], opts["id"])

    def member_of_contactgroups(self, opts):
        ret = H.model.db.contactgroup.query(
            query=H.req.get_query_data(),
            count="id",
            table=["contactgroup", ["id", "name", "description"]],
            join=[
                "inner", [
                    "host_contactgroup",
                    "contactgroup.id",
                    "host_contactgroup.contactgroup_id"
                ]
            ],
            condition=["where", ["host_contactgroup.host_id", opts["id"]]],
            order=["asc", "contactgroup.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def no_member_of_contactgroups(self, opts):
        ret = H.model.db.contactgroup.query(
            query=H.req.get_query_data(),
            count="id",
            table=["contactgroup", ["id", "name", "description"]],
            condition=[
                "where", ["contactgroup.id", "not in", {
                    "table": "contactgroup",
                    "columns": "id",
                    "join": [
                        "inner", [
                            "host_contactgroup",
                            "contactgroup.id",
                            "host_contactgroup.contactgroup_id"
                        ],
                    ],
                    "condition": [
                        "where", ["host_contactgroup.host_id", opts["id"]],
                    ]
                }]
            ],
            order=["asc", "contactgroup.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def add_to_contactgroups(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_contactgroup.create({
            "contactgroup_id": opts["contactgroup_id"],
            "host_id": opts["id"]
        });

    def remove_from_contactgroups(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_contactgroup.delete_by_col(
            "contactgroup_id", opts["contactgroup_id"],
            "host_id", opts["id"],
        )

    def member_of_groups(self, opts):
        ret = H.model.db.group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["group", ["id", "name", "description"]],
            join=[
                "inner", [
                    "host_group",
                    "group.id",
                    "host_group.group_id"
                ]
            ],
            condition=["where", ["host_group.host_id", opts["id"]]],
            order=["asc", "group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def no_member_of_groups(self, opts):
        ret = H.model.db.group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["group", ["id", "name", "description"]],
            condition=[
                "where", ["group.id", "not in", {
                    "table": "group",
                    "columns": "id",
                    "join": [
                        "inner", ["host_group", "group.id", "host_group.group_id"],
                    ],
                    "condition": [
                        "where", ["host_group.host_id", opts["id"]],
                    ]
                }]
            ],
            order=["asc", "group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def add_to_groups(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_group.create({
            "group_id": opts["group_id"],
            "host_id": opts["id"]
        });

    def remove_from_groups(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_group.delete_by_col(
            "group_id", opts["group_id"],
            "host_id", opts["id"],
        )

    def member_of_location_groups(self, opts):
        ret = H.model.db.location_group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["location_group", ["id", "name", "description"]],
            join=[
                "inner", [
                    "host_location_group",
                    "location_group.id",
                    "host_location_group.location_group_id"
                ]
            ],
            condition=["where", ["host_location_group.host_id", opts["id"]]],
            order=["asc", "location_group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def no_member_of_location_groups(self, opts):
        ret = H.model.db.location_group.query(
            query=H.req.get_query_data(),
            count="id",
            table=["location_group", ["id", "name", "description"]],
            condition=[
                "where", ["location_group.id", "not in", {
                    "table": "location_group",
                    "columns": "id",
                    "join": [
                        "inner", [
                            "host_location_group",
                            "location_group.id",
                            "host_location_group.location_group_id"
                        ],
                    ],
                    "condition": [
                        "where", ["host_location_group.host_id", opts["id"]],
                    ]
                }]
            ],
            order=["asc", "location_group.name"]
        )

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def add_to_location_groups(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_location_group.create({
            "location_group_id": opts["location_group_id"],
            "host_id": opts["id"]
        });

    def remove_from_location_groups(self, opts):
        if not H.plugin.token.check():
            return
        H.model.db.host_location_group.delete_by_col(
            "location_group_id", opts["location_group_id"],
            "host_id", opts["id"],
        )
