from bloonix.heaven import H
from bloonix.heaven.plugin.crud import CRUD
from bloonix.heaven.validator import Validator
import os

class Controller(CRUD):
    def startup(self):
        self.db = H.model.db.message_service
        self.list_opts = {
            "parse_int": ["id", "enabled"],
            "count": "id",
            "table": "message_service",
            "columns": ["id", "name", "enabled"],
            "order": ["asc", "message_service.name"]
        }
        path = "/config/message-services"
        builtin = ":builtin(sendmail|http|script)"
        H.route(path, "list")
        H.route("%s/:id" % path, "show")
        H.route("%s/create/%s" % (path, builtin), "create_or_update")
        H.route("%s/:id/update" % path, "create_or_update")
        H.route("%s/:id/delete" % path, "delete")
        H.route("%s/options/%s" % (path, builtin), "options")
        H.route("%s/:id/options" % path, "options")

    def show(self, opts):
        H.stash.data = self._split_builtin_params(H.cache["message_service"])

    def create_or_update(self, opts):
        # ** DATA **
        #
        # sendmail
        #   'name': 'foo',
        #   'enabled': 1,
        #   'message_template': 'bar',
        #   'sender': 'bloonix@localhost',
        #   'send_to': '%send_to%',
        #   'subject': 'Bloonix-Monitoring Status Message',
        #   'message': '%message%'
        #
        # script
        #   'name': 'foo',
        #   'enabled': 1,
        #   'message_template': 'bar',
        #   'script': 'baz',
        #   'params': {'param1': 'value1', 'param2': 'value2'}
        #
        if not H.plugin.token.check():
            return
        if "id" in opts:
            builtin = H.cache["message_service"]["builtin"]
            ignore_missing = True
        else:
            builtin = opts["builtin"]
            ignore_missing = None

        validator = self.db.validator[builtin]
        res = validator.validate(ignore_missing=ignore_missing)
        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        table_data = {}
        for c in ["name", "enabled", "message_template", "script", "params"]:
            if c in res.data:
                table_data[c] = res.data[c]
                del res.data[c]

        if builtin != "script":
            table_data["params"] = res.data

        if "id" in opts:
            self.db.update_by_id(opts["id"], table_data)
            ret = self.db.get(opts["id"])
        else:
            table_data["builtin"] = builtin
            ret = self.db.create_and_get(table_data)

        H.plugin.audit.update(
            target=self.db.table,
            old_data=H.cache.get(self.db.table, None),
            new_data=ret)
        H.stash.data = ret

    def options(self, opts):
        if "id" in opts:
            builtin = H.cache["message_service"]["builtin"]
        else:
            builtin = opts["builtin"]

        super().options(opts, validator=self.db.validator[builtin])

        if "id" in opts:
            H.stash.data["values"] = self._split_builtin_params(
                H.stash.data["values"])

    def _split_builtin_params(self, data):
        if data["builtin"] != "script":
            for k, v in data["params"].items():
                data[k] = v
            del data["params"]
        return data
