from bloonix.heaven import H
from bloonix.heaven.validator import Validator
import bloonix.rx as rx
import bloonix.utils as utils
import math
import re
import time

class Controller:
    def startup(self):
        self.db = H.model.db.host
        H.route("/hosts", "list")
        H.route("/hosts/directory/:cls(project|location)", "dir")
        H.route("/hosts/:id", "show")
        H.route("/hosts/:id/events", "events")
        H.route("/hosts/:id/dependencies", "dependencies")
        H.route("/hosts/:id/notifications", "notifications")
        H.route("/hosts/:id/services", "services")
        H.route("/hosts/:id/services/:service_id", "service")
        H.route("/hosts/:id/services/:service_id/plugin", "get_plugin")
        H.route("/hosts/:id/host-downtimes", "host_downtimes")
        H.route("/hosts/:id/service-downtimes", "service_downtimes")
        H.route("/hosts/:id/count-events", "count_events")
        H.route("/hosts/:id/count-flapping", "count_flapping")
        H.route("/hosts/:id/count-downtime", "count_downtime")
        H.route("/hosts/:id/count-notifications", "count_notifications")
        H.route("/hosts/:id/count-not-ok", "count_not_ok")
        H.route("/hosts/:id/latest-availability", "latest_availability")

    def auto(self, opts):
        if "id" in opts:
            host = self.db.by_id(opts["id"])
            if not host:
                return H.plugin.error.set("err-600")
            H.cache["host"] = host
        if "service_id" in opts:
            service = H.model.db.service.by_id_and_by_host_id(
                opts["service_id"], opts["id"])
            if not service:
                return H.plugin.error.set("err-600")
            H.cache["service"] = service
        return True

    def list(self, opts):
        ret = self.db.list(H.req.get_query_data())

        if ret.success == False:
            return H.plugin.error.set("err-606")

        for host in ret.data:
            host["status_count"] = {}
            for row in H.model.db.service.status_by_host_id(host["id"]):
                host["status_count"][row["status"]] = row["count"]
            
        H.stash.data = ret.data
        H.stash.total = ret.total

    def show(self, opts):
        H.stash.data = H.cache["host"]

    def dir(self, opts):
        cls = "%s_class" % opts["cls"]
        rows = H.model.db.host.dir(cls)
        H.stash.data = []
        for row in rows:
            parts = re.split("/", row[cls])
            part = ""
            while len(parts):
                p = parts.pop(0)
                if p == "":
                    continue
                part += "/%s" % p
                if part not in H.stash.data:
                    H.stash.data.append(part)

    def services(self, opts):
        ret = H.model.db.service.list(H.req.get_query_data(), opts["id"])

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def service(self, opts):
        H.stash.data = H.model.db.service.by_id(opts["service_id"])

    def get_plugin(self, opts):
        H.stash.data = H.model.db.plugin.by_command(H.cache["service"]["command"])

    def host_downtimes(self, opts):
        H.stash.data = H.model.db.host_downtime.by_host_id(opts["id"])
        H.stash.total = len(H.stash.data)

    def service_downtimes(self, opts):
        H.stash.data = H.model.db.service_downtime.by_host_id(opts["id"])
        H.stash.total = len(H.stash.data)

    def events(self, opts):
        res = H.model.db.event.validate()

        if res.has_failed:
            return H.plugin.error.set("err-610", failed=res.failed)

        ret = H.model.db.event.list(
            H.req.get_query_data(),
            res.data["date"],
            opts["id"])

        if ret.success == False:
            return H.plugin.error.set("err-606")

        H.stash.data = ret.data
        H.stash.total = ret.total

    def dependencies(self, opts):
        i = 0
        d = {
            "name": "",
            "children": []
        }
        s = {}
        rows = H.model.db.dependency.get_service_ids(opts["id"])
        for row in rows:
            d["children"].append(self.__get_dep(row["service_id"], i, s))
        H.stash.data = {
            "tree": d,
            "services": s,
            "host": {
                "id": H.cache["host"]["id"],
                "hostname": H.cache["host"]["hostname"],
            }
        }

    def __get_dep(self, service_id, i, s):
        i += 1
        row = H.model.db.service.service_dep_data(service_id)
        s[service_id] = row
        d = {
            "name": "%s (%s)" % (row["service_name"], service_id),
            "value": service_id,
            "children": []
        }
        if i < 10:
            rows = H.model.db.dependency.get_on_service_id(service_id)
            for row in rows:
                d["children"].append(self.__get_dep(row["on_service_id"], i, s))
        return d

    def notifications(self, opts):
        if H.user["role"] not in ["admin", "operator"]:
            return H.plugin.error.set("err-605")
        ret = H.model.db.notification.list_by_host_id(H.req.get_query_data(), opts["id"])
        if ret.success == False:
            return H.plugin.error.set("err-606")
        H.stash.data = ret.data
        H.stash.total = ret.total

    def count_events(self, opts):
        H.stash.data = H.model.db.event.count_latest_by_host_id(opts["id"])

    def count_downtime(self, opts):
        H.stash.data = H.model.db.service.count_downtime_by_host_id(opts["id"])

    def count_flapping(self, opts):
        H.stash.data = H.model.db.service.count_flapping_by_host_id(opts["id"])

    def count_notifications(self, opts):
        H.stash.data = H.model.db.notification.count_by_host_id(opts["id"])

    def count_not_ok(self, opts):
        H.stash.data = H.model.db.service.count_not_ok_by_host_id(opts["id"])

    def latest_availability(self, opts):
        services = H.model.db.service.get_status_since(opts["id"])
        events = H.model.db.event.latest_by_host_id(opts["id"])
        cur_time = int(time.time())
        last_status = {}
        av = {}
        total = 0

        for service in services:
            av[service["id"]] = 0
            last_status[service["id"]] = [service["status"], cur_time]
            if service["status"] == 5:
                av[service["id"]] += cur_time - service["status_since"]

        for event in events:
            last_status[service["id"]] = [event["last_status"], event["time"]]
            if event["last_status"] == 5:
                av[event["service_id"]] += event["duration"]

        for service_id in av:
            if last_status[service_id][0] == 5:
                av[service_id] += int(time.time()) - last_status[service_id][1]

            if av[service_id] >= 86400:
                total += 0
            else:
                total += 100 - (av[service_id] * 100 / 86400)

        if len(services) > 0:
            H.stash.data = "%.2f" % (total / len(services))
        else:
            H.stash.data = 0.0
