from bloonix.heaven import H
from bloonix import utils
import json
import re
import time

class Controller:
    def startup(self):
        H.route("/", "index")
        H.route("/robots.txt", "robots")
        H.route("/ping", "ping")

    def auto(self, opts):
        H.model.db.dbi.connect()
        H.model.ds.dbi.connect()
        lang = H.req.cookie("lang")
        if lang not in ["en", "de"]:
            lang = "en"

        H.lang.set(lang)
        H.maintenance = H.model.db.sys_options.has_maintenance()
        H.render("json")

        if H.process in ["root.index", "login.login"]:
            nonce = utils.pwgen(20)
            H.stash.data["meta"] = {
                "version": H.version,
                "lang": H.lang.active_lang,
                "time": time.time(),
                "nonce": nonce,
                "style": "default",
            }
            for pair in H.config["add_header"]:
                H.log.notice(pair)
                m = re.search(r"^([^\s]+)\s+(.+)", pair)
                if m:
                    param, value = m.group(1), m.group(2)
                    if (param == "Content-Security-Policy"
                            and re.search(r"nonce-%s", value)):
                        value = re.sub(r"nonce-%s", "nonce-%s" % nonce, value)
                    H.res.header(param, value)

        if H.process in [
                "root.robots",
                "root.ping",
                "login.login",
                "register.register"]:
            return True

        if self.check_auth() == True:
            if (H.session["2fa_success"] == 0
                    and H.process != "login.two_fa_auth"
                    and H.process != "login.logout"
                    and H.process != "token.csrf"):
                if H.req.is_json:
                    return H.plugin.error.set("err-637")
                if H.process != "root.index":
                    H.res.redirect("/")
                    H.plugin.error.set("err-302")
                    return False
            if (H.user["force_password_change"] == 1
                    and H.process != "login.logout"):
                # If the user has not changed the password,
                # so pass all actions that are necessary
                # to change the password.
                if H.process in [
                        "user.password_options",
                        "user.set_password",
                        "token.csrf"]:
                    return True

                if H.req.is_json:
                    return H.plugin.error.set("err-660")

                if H.process != "root.index":
                    H.res.redirect("/")
                    H.plugin.error.set("err-302")
                    return False
            return True

        if H.req.is_json:
            H.plugin.error.set("err-630")
        else:
            H.res.redirect("/login")
            H.plugin.error.set("err-302")

        return False

    def check_auth(self):
        sid = H.req.param("sid")
        if not sid:
            sid = H.req.cookie("sid")
            if not sid:
                return False

        H.log.info("check if sid exist")
        session = H.model.db.user_session.find("sid", sid)

        if not session or session["expire"] <= H.time:
            return False

        user = H.model.db.user.get(session["user_id"])

        if not user or user["locked"] == 1:
            return False

        if (H.time
                + H.config["session_expire_time"]
                - session["expire"]
                > H.config["session_refresh_time"]):
            H.res.cookie("sid", sid, H.config["session_expire_time"])
            H.model.db.user_session.update_by_col(
                "sid", sid, {"expire": H.time + H.config["session_expire_time"]}
            )

        H.session_id = sid
        H.session = session
        H.user = user
        H.log.set_pattern("%Y", user["username"])
        return True

    # called by H if no route match
    def default(self, opts):
        H.plugin.error.set("err-404")
        H.res.status = "404 Not Found"

    # called by H if an error occurs
    def error(self, opts):
        H.plugin.error.set("err-500")
        H.res.status = "500 Internal Server Error"

    def index(self, opts):
        H.stash.template = "index.html"
        H.stash.data["meta"]["style"] = H.user.get("style", "default")
        #H.stash.data["meta"]["highcharts"] = H.config.get("highcharts", None)
        H.render("jinja")

    def ping(self, opts):
        H.stash.data = "pong"

    def robots(self, opts):
        H.res.content = "User-agent: *\nDisallow: /\n"
        H.render("plain")

    def end(self, opts):
        H.model.db.dbi.disconnect()
        H.model.ds.dbi.disconnect()
        runtime = time.time() - H.time
        H.req.path_info()
        H.log.notice("finished request %s (took %ss)" % (H.req.path_info(), runtime))
        return True
