from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix.heaven import H
from bloonix import mstatus
from bloonix import timeperiod

class ContactMessageService(CRUDV):
    def init(self):
        notification_level_default = [0, 2, 3, 4, 5, 6]
        notification_level = []

        for c in [0, 2, 3, 4, 5, 6]:
            notification_level.append({"label": mstatus.STRING[c], "value": c})

        self.validator = Validator()
        self.validator.set({
            "send_to": {
                "type": str,
                "minlength": 1,
                "maxlength": 200
            },
            "message_service_id": {
                "type": int,
                "list_of_dicts_key": "value",
                "list_of_dicts_options": self.schema.message_service.form_data("name", "id")
            },
            "notification_level": {
                "type": list,
                "list_of_dicts_key": "value",
                "list_of_dicts_multioptions": notification_level,
                "default": notification_level_default
            },
            "timeperiods": {
                "type": list,
                "function": timeperiod.validate,
                "default": ["Monday - Sunday 00:00 - 23:59"],
                "maxsize": 20
            },
            "enabled": {
                "type": int,
                "options": [0,1],
                "default": 1
            }
        })

    def by_contact_id(self, contact_id):
        stmt, bind = self.sql.select(
            table=[
                "contact_message_service", "*",
                "message_service", "name"
            ],
            join=[
                "inner", [
                    "message_service",
                    "contact_message_service.message_service_id",
                    "message_service.id"
                ]
            ],
            condition=[
                "where", [
                    "contact_message_service.contact_id", contact_id
                ]
            ],
            order=[
                "asc", "message_service.name"
            ]
        )
        return self.dbi.fetchall(stmt, bind)

    def by_id_and_contact_id(self, cms_id, contact_id):
        stmt, bind = self.sql.select(
            table="contact_message_service",
            columns="*",
            condition=[
                "where", ["contact_message_service.id", cms_id],
                "and", ["contact_message_service.contact_id", contact_id]
            ])
        return self.dbi.fetchone(stmt, bind)
