from bloonix.heaven import H
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix import ccodes
from bloonix import rx
from bloonix import utils
from bloonix.netaddr import validate_ip
import re

class Host(CRUDV):
    def __init__(self):
        self.has_unique_column = {
            "column": "hostname",
            "err": "err-618",
        }

    def init(self, action, genpw=None):
        #H.log.notice(self.schema.template.form_data("name", "id"), dump=True)
        #H.log.notice(self.schema.contactgroup.form_data("name", "id"), dump=True)
        #H.log.notice(self.schema.group.form_data("name", "id"), dump=True)
        #H.log.notice(self.schema.location_group.form_data("name", "id"), dump=True)

        self.validator = Validator(post_validation=self.__post_validation)
        if action == "create":
            allow_empty_pw = False
            self.validator.set({
                "template_id": {
                    "type": list,
                    "list_of_dicts_key": "value",
                    "list_of_dicts_multioptions": self.schema.template.form_data("name", "id"),
                    "allow_empty": True,
                    "default": [],
                }
            })
            self.validator.set({
                "contactgroup_id": {
                    "type": list,
                    "list_of_dicts_key": "value",
                    "list_of_dicts_multioptions": self.schema.contactgroup.form_data("name", "id"),
                    "allow_empty": True,
                    "default": [],
                }
            })
            self.validator.set({
                "group_id": {
                    "type": list,
                    "list_of_dicts_key": "value",
                    "list_of_dicts_multioptions": self.schema.group.form_data("name", "id"),
                    "allow_empty": True,
                    "default": [],
                }
            })
            self.validator.set({
                "location_group_id": {
                    "type": list,
                    "list_of_dicts_key": "value",
                    "list_of_dicts_multioptions": self.schema.location_group.form_data("name", "id"),
                    "allow_empty": True,
                    "default": [],
                }
            })
        else:
            allow_empty_pw = True

        self.validator.set({
            "hostname": {
                "type": str,
                "regex": rx.HOSTNAME
            },
            "ipaddr1": {
                "type": str,
                "regex": rx.IPADDR
            },
            "ipaddr2": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr3": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "ipaddr3": {
                "type": str,
                "regex": rx.IPADDR,
                "allow_empty": True,
                "optional": True
            },
            "password": {
                "type": str,
                "regex": r"^(?:bloonix|[^\s]{30,512})\Z",
                "default": utils.pwgen(40)
            },
            "allow_from": {
                "type": list,
                "allow_empty": True,
                "maxsize": 10,
                "maxlength": 500,
                #"default": ["%IPADDR1%","%IPADDR2%","%IPADDR3%","%IPADDR4%"],
            },
            "description": {
                "type": str,
                "minlength": 0,
                "maxlength": 100,
                "optional": True
            },
            "sysinfo": {
                "type": str,
                "minlength": 0,
                "maxlength": 400,
                "optional": True
            },
            "comment": {
                "type": str,
                "minlength": 0,
                "maxlength": 400,
                "optional": True
            },
            "country_code": {
                "type": str,
                "list_of_dicts_key": "value",
                "list_of_dicts_options": ccodes.country_as_form(),
                "allow_empty": True,
                "optional": True
            },
            "location_class": {
                "type": str,
                "regex": rx.CLASS_PATH,
                "maxlength": 100,
                "default": "",
            },
            "project_class": {
                "type": str,
                "regex": rx.CLASS_PATH,
                "maxlength": 100,
                "default": "",
            },
            "enabled": {
                "type": int,
                "options": [0,1],
                "default": 1
            },
            "notification_enabled": {
                "type": int,
                "options": [0,1],
                "default": 1
            },
            "notification_interval": {
                "type": int,
                "options": [
                    300, 900, 1800, 2700, 3600, 7200, 14400,
                    28800, 43200, 57600, 86400, 172800],
                "default": 1800
            },
            "variables": {
                "type": dict,
                "function": utils.validate_pv_dict,
                "allow_empty": True,
                "maxlength": 20000,
                "default": {}
            }
        })

    def __post_validation(self, data):
        if "allow_from" in data:
            for ip in data["allow_from"]:
                if (not re.search(r"^%IPADDR[1-4]?%\Z", ip)
                        and not validate_ip(ip)):
                    return {"allow_from": "err-711"}

    def by_id(self, host_id):
        select = H.plugin.role.filter_hosts(
            table=[
                "host", [
                    "id",
                    "hostname",
                    "ipaddr1",
                    "ipaddr2",
                    "ipaddr3",
                    "ipaddr4",
                    "description",
                    "comment",
                    "sysinfo",
                    "enabled",
                    "notification_enabled",
                    "notification_interval",
                    "country_code",
                    "location_class",
                    "project_class",
                    "facts",
                ],
            ],
            condition=[
                "where", ["host.id", host_id]
            ])

        stmt, bind = self.sql.select(**select)
        return self.dbi.fetchone(stmt, bind)

    def list(self, query_data):
        query = H.plugin.role.filter_hosts(
            distinct=True,
            query=query_data,
            count="id",
            parse_int=[
                "id",
                "enabled",
                "notification_interval",
                "notification_enabled",
            ],
            table=[
                "host", [
                    "id",
                    "hostname",
                    "description",
                    "ipaddr1",
                    "ipaddr2",
                    "ipaddr3",
                    "ipaddr4",
                    "country_code",
                    "location_class",
                    "project_class",
                    "enabled",
                    "notification_interval",
                    "notification_enabled",
                ],
                "service", [
                    {
                        "func": "coalesce",
                        "default": 1,
                        "as": "status",
                        "value": {
                            "func": "max",
                            "column": "status",
                        },
                    }
                ]
            ],
            join=[
                "-left", [
                    "service",
                    "host.id", 
                    "service.host_id"
                ],
            ],
            group=[
                "host.id"
            ],
            order=[
                "desc", "status",
                "asc", "host.hostname"
            ]
        )

        return self.query(**query)

    def dir(self, by):
        if by not in ["project_class", "location_class"]:
            raise Exception("invalid class")

        query = H.plugin.role.filter_hosts(
            distinct=True,
            table=["host", by],
            order=["asc", by],
        )

        stmt, bind = self.sql.select(**query)
        return self.dbi.fetchall(stmt, bind)

    def get_all_variables(self, host_id):
        # distinct() on table template raises an database error:
        #   could not identify an equality operator for type json
        # for this reason the statemets are splitted
        subselect = {
            "table": [
                "service_parameter", [
                    "distinct(template_id)"
                ],
            ],
            "join": [
                "inner", [
                    "service",
                    "service_parameter.id",
                    "service.service_parameter_id",
                ],
            ],
            "condition": [
                "where", [
                    "service.host_id", "=", host_id
                ],
                "and", [
                    "service_parameter.template_id", "is not", "null"
                ]
            ]
        }

        stmt, bind = self.sql.select(
            table=[
                "template", ["id", "name", "variables"]
            ],
            condition=[
                "where", ["id", "in", subselect]
            ]
        )
        return self.dbi.fetchall(stmt, bind)

    def by_hostname(self, hostname):
        return self.fetchone(condition=["where", ["hostname", hostname]])
