from bloonix.dbi.crud import CRUDV
from bloonix.heaven import H
from bloonix.exception import BloonixTooMuchObjects

class Service(CRUDV):
    def check_count_services(self, plus=0, wtrm=None):
        if wtrm == True:
            # First, it is checked whether bloonix_max_services
            # is not exceeded.
            self.__check_count_services(plus=plus, wtrm=None)
        self.__check_count_services(plus=plus, wtrm=wtrm)

    def __check_count_services(self, plus=0, wtrm=None):
        stmt = {"count": "service.id"}
        if wtrm == True and H.config["bloonix_max_wtrm_services"] > 0:
            max_services = H.config["bloonix_max_wtrm_services"]
            error_msg = "the limit of %s web transactions is exceeded" % max_services
            stmt["join"] = [
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ]
            ]
            stmt["condition"] = [
                "where", [
                    "service_parameter.command", "check-wtrm"
                ]
            ]
        else:
            max_services = H.config["bloonix_max_services"]
            error_msg = "the limit of %s services is exceeded" % max_services
        if max_services > 0:
            count = self.schema.service.count(**stmt)
            if count + plus >= max_services:
                raise BloonixTooMuchObjects(error_msg)

    def status(self, host=None):
        query = {
            "table": ["service", ["status"]],
            "count": "status",
            "group": "status"
        }
        if H.plugin.role.has_filter_by_group():
            subquery = H.plugin.role.filter_services(
                distinct=True,
                table=[
                    "service", ["id"],
                ]
            )
            query["condition"] = [
                "where", ["service.id", "in", subquery]
            ]
        stmt, bind = self.sql.select(**query)
        return self.dbi.fetchall(stmt, bind)

    def status_by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=["service", ["status"]],
            count="status",
            group="status",
            condition=["where", ["host_id", host_id]]
        )
        return self.dbi.fetchall(stmt, bind)

    def by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=[
                "service", [
                    "id",
                    "service_parameter_id",
                    "status",
                    "enabled",
                    "notification_enabled",
                    "acknowledged"
                ],
                "service_parameter", [
                    "satellite_type",
                    "template_id",
                    "service_name",
                    "command",
                    "remote_check",
                    "interval",
                    "retry_interval",
                    "attempt_max",
                    "host_alive_check"
                ]
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ]
            ],
            condition=["where", ["service.host_id", host_id]],
            order=["asc", "service_name"]
        )
        return self.dbi.fetchall(stmt, bind)

    def service_dep_data(self, service_id):
        stmt, bind = self.sql.select(
            table=[
                "service", "host_id",
                "service_parameter", "service_name",
                "host", "hostname",
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id"
                ]
            ],
            condition=[
                "where", ["service.id", service_id]
            ],
        )
        return self.dbi.fetchone(stmt, bind)

    def by_id(self, service_id):
        select = H.plugin.role.filter_services(
            table=[
                "service", [
                    "id", "status",
                ],
                "service_parameter", [
                    "service_name", "command",
                ],
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ]
            ],
            condition=[
                "where", ["service.id", service_id]
            ],
        )
        stmt, bind = self.sql.select(**select)
        return self.dbi.fetchone(stmt, bind)

    def by_id_with_hostname(self, service_id):
        select = H.plugin.role.filter_services(
            table=[
                "service", [
                    "id", "status",
                ],
                "service_parameter", [
                    "service_name", "command", "interval",
                ],
                "host", [
                    "hostname",
                ],
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id",
                ],
            ],
            condition=[
                "where", ["service.id", service_id]
            ],
        )
        stmt, bind = self.sql.select(**select)
        return self.dbi.fetchone(stmt, bind)

    def by_id_and_by_host_id(self, service_id, host_id):
        select = H.plugin.role.filter_services(
            table=[
                "service", [
                    "id", "status",
                ],
                "service_parameter", [
                    "service_name", "command",
                ],
                "host", [
                    "hostname",
                ],
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id",
                ],
            ],
            condition=[
                "where", ["service.id", service_id],
                "and", ["service.host_id", host_id],
            ],
        )
        stmt, bind = self.sql.select(**select)
        return self.dbi.fetchone(stmt, bind)

    def ids_by_host_id(self, host_id):
        service_ids = []
        stmt, bind = self.sql.select(
            table=[
                "service", [
                    "id",
                ],
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id"
                ]
            ],
            condition=["where", ["host.id", host_id]],
            order=["asc", "service_name"]
        )
        for row in self.dbi.fetchall(stmt, bind):
            service_ids.append(row["id"])
        return service_ids

    def list(self, query_data, host_id=None):
        condition = []
        if host_id is not None:
            condition.append("where")
            condition.append(["service.host_id", host_id])
        query = H.plugin.role.filter_services(
            distinct=True,
            query=query_data,
            parse_int=[
                "id", "host_id", "status", "status_since",
                "last_check", "attempt_counter", "acknowledged",
                "enabled", "notification_enabled", "attempt_max",
                "is_flapping", "has_downtime", "remote_check",
                "host_enabled", "host_notification_enabled",
                "service_parameter_id",
            ],
            count="id",
            table=[
                "service", [
                    "id",
                    "service_parameter_id",
                    "host_id",
                    "status",
                    "status_since",
                    "tags",
                    "message",
                    "last_check",
                    "attempt_counter",
                    "acknowledged",
                    "enabled",
                    "notification_enabled",
                    "is_flapping",
                    "has_downtime",
                    "agent_version",
                ],
                "service_parameter", [
                    "satellite_type",
                    "template_id",
                    "service_name",
                    "attempt_max",
                    "command",
                    "remote_check",
                ],
                "host", [
                    "hostname",
                    "ipaddr1",
                    "project_class",
                    "location_class",
                    "enabled AS host_enabled",
                    "notification_enabled AS host_notification_enabled",
                ]
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id"
                ]
            ],
            condition=condition,
            order=[
                "desc", "service.status"
            ]
        )
        ret = self.query(**query)
        if H.user["role"] == "admin" or H.user["role"] == "operator":
            template_ids = []
            for service in ret.data:
                if service["template_id"]:
                    template_ids.append(service["template_id"])
            if len(template_ids):
                templates = H.model.db.template.get_names_by_ids(template_ids)
                for service in ret.data:
                    if service["template_id"]:
                        service["template_name"] = templates[service["template_id"]]
        return ret

    def short(self, query_data):
        query = H.plugin.role.filter_services(
            distinct=True,
            query=query_data,
            parse_int=["id", "host_id", "status", "acknowledged"],
            table=[
                "service", [
                    "id",
                    "host_id",
                    "status",
                    "message",
                    "acknowledged",
                ],
                "service_parameter", [
                    "service_name",
                ],
                "host", [
                    "hostname",
                    "ipaddr1",
                    "project_class",
                    "location_class",
                ]
            ],
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ],
                "inner", [
                    "host",
                    "service.host_id",
                    "host.id"
                ]
            ],
            order=[
                "desc", "service.status",
                "asc", "service.host_id",
            ]
        )

        return self.query(**query)

    def count_downtime_by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            count="id",
            table="service",
            condition=[
                "where", ["host_id", host_id],
                "and", ["has_downtime", 1],
            ])
        return self.dbi.fetchone(stmt, bind)["count"]

    def count_flapping_by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            count="id",
            table="service",
            condition=[
                "where", ["host_id", host_id],
                "and", ["is_flapping", 1],
            ])
        return self.dbi.fetchone(stmt, bind)["count"]

    def count_not_ok_by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            count="id",
            table="service",
            condition=[
                "where", ["host_id", host_id],
                "and", ["status", ">", 1],
            ])
        return self.dbi.fetchone(stmt, bind)["count"]

    def get_status_since(self, host_id, service_id=None):
        condition = ["where"]
        if service_id:
            condition.append(["service.id", service_id])
        else:
            condition.append(["service.host_id", host_id])
        stmt, bind = self.sql.select(
            table=[
                "service", [
                    "id",
                    "host_id",
                    "status",
                    "status_since",
                ],
                "service_parameter", "service_name",
            ],
            condition=condition,
            join=[
                "inner", [
                    "service_parameter",
                    "service.service_parameter_id",
                    "service_parameter.id"
                ]
            ],
            order=["asc", "service_parameter.service_name"]
        )
        return self.dbi.fetchall(stmt, bind)
