import time
from bloonix.dbi.crud import CRUDV
from bloonix.heaven.validator import Validator
from bloonix import timeperiod

class ServiceDowntime(CRUDV):
    def by_host_id(self, host_id):
        stmt, bind = self.sql.select(
            table=[
                "service_downtime", "*",
                "service_parameter", "service_name"
            ],
            join=[
                "inner", ["service", "service_downtime.service_id", "service.id"],
                "inner", ["service_parameter", "service.service_parameter_id", "service_parameter.id"]
            ],
            condition=[
                "where", ["service_downtime.host_id", host_id]
            ],
            order=["asc", "service_parameter.service_name"]
        )
        rows = self.dbi.fetchall(stmt, bind)
        unixtime = time.time()
        for row in rows:
            row["active"] = 0
            if len(row["timeslice"]):
                for timeslice in row["timeslice"]:
                    if timeperiod.check(timeslice):
                        row["active"] = 1
                        break
            elif row["begin"] < unixtime and row["end"] > unixtime:
                row["active"] = 1
        return rows

    def by_id_and_host_id(self, service_downtime_id, host_id):
        stmt, bind = self.sql.select(
            table="service_downtime",
            columns="*",
            condition=[
                "where", ["id", service_downtime_id],
                "and", ["host_id", host_id]
            ])
        return self.dbi.fetchone(stmt, bind)
